var helpers = require('./helpers');
var webpackMerge = require('webpack-merge');
var commonConfig = require('./webpack.common.js');

// Webpack Plugins
const AddAssetHtmlPlugin = require('add-asset-html-webpack-plugin');
const DefinePlugin = require('webpack/lib/DefinePlugin');
const NamedModulesPlugin = require('webpack/lib/NamedModulesPlugin');
const LoaderOptionsPlugin = require('webpack/lib/LoaderOptionsPlugin');

//# Webpack Constants
const ENV = process.env.ENV = process.env.NODE_ENV = 'development';
const HMR = helpers.hasProcessFlag('hot');
const METADATA = {
  title: 'SOTS',
  baseUrl: '/',
  host: process.env.HOST || 'localhost',
  port: process.env.PORT || 8080,
  ENV: ENV,
  HMR: HMR
};

//# Environment Config Object
var envConfig = require('./config.json');
var clientConfig = webpackMerge(commonConfig.CLIENT, {
  devtool: 'cheap-module-source-map',

  output: {
    path: helpers.root('dist'),
    filename: '[name].bundle.js',
    sourceMapFilename: '[name].bundle.map',
    chunkFilename: '[id].chunk.js'
  },

  devServer: {
    proxy: {
      '*': {
        target: 'http://localhost:' + envConfig.PORT,
        secure: false
      }
    },
    port: METADATA.port,
    host: METADATA.host,
    historyApiFallback: true,
    watchOptions: {
      aggregateTimeout: 300,
      poll: 1000
    }
  },

  plugins: [
    new DefinePlugin({
      'ENV': JSON.stringify(METADATA.ENV),
      'HMR': METADATA.HMR,
      'process.env': {
        'ENV': JSON.stringify(METADATA.ENV),
        'NODE_ENV': JSON.stringify(METADATA.ENV),
        'HMR': METADATA.HMR
      }
    }),
  
    new LoaderOptionsPlugin({
      debug: true,
      options: {
        metadata: METADATA,
        tslint: {
          emitErrors: false,
          failOnHint: false,
          resourcePath: 'src',
        },

        node: {
          global: true,
          crypto: 'empty',
          process: true,
          module: false,
          clearImmediate: false,
          setImmediate: false
        }
      }
    })
  ]
});

var serverConfig = webpackMerge(commonConfig.SERVER, {
  devtool: 'sourcemap',
  output: {
    path: helpers.root('dist'),
    filename: '[name].api.bundle.js',
    sourceMapFilename: '[name].api.bundle.map',
    chunkFilename: '[id].api.chunk.js',
    libraryTarget: 'commonjs'
  }
});

module.exports = [clientConfig, serverConfig];