var helpers = require('./helpers');
var webpackMerge = require('webpack-merge');
var commonConfig = require('./webpack.common.js');

// Webpack Plugins
const ProvidePlugin = require('webpack/lib/ProvidePlugin');
const DefinePlugin = require('webpack/lib/DefinePlugin');
const UglifyJsPlugin = require('webpack/lib/optimize/UglifyJsPlugin');
const LoaderOptionsPlugin = require('webpack/lib/LoaderOptionsPlugin');
const CompressionPlugin = require('compression-webpack-plugin');
const WebpackMd5Hash = require('webpack-md5-hash');
const CommonsChunkPlugin = require('webpack/lib/optimize/CommonsChunkPlugin');

//# Webpack Constants
const ENV = process.env.ENV = process.env.NODE_ENV = 'production';
const METADATA = {
  title: 'SOTS',
  baseUrl: '/',
  host: process.env.HOST || '0.0.0.0',
  port: process.env.PORT || 8080,
  ENV: ENV,
  HMR: false
};

//# Common Plugins
const commonPlugins = [
  new WebpackMd5Hash(),
  new UglifyJsPlugin({
    beautify: false,
    mangle: {
      screw_ie8: true,
      keep_fnames: true
    },
    compress: {
      screw_ie8: true
    },
    comments: false
  })
];

//# Environment Config Object
var envConfig = require('./config.json');
var clientConfig = webpackMerge(commonConfig.CLIENT, {
  devtool: 'source-map',

  output: {
    path: helpers.root('dist'),
    filename: '[name].[chunkhash].bundle.js',
    sourceMapFilename: '[name].[chunkhash].bundle.map',
    chunkFilename: '[id].[chunkhash].chunk.js'
  },

  plugins: [
    ...commonPlugins,
    new DefinePlugin({
      'ENV': JSON.stringify(METADATA.ENV),
      'HMR': METADATA.HMR,
      'process.env': {
        'ENV': JSON.stringify(METADATA.ENV),
        'NODE_ENV': JSON.stringify(METADATA.ENV),
        'HMR': METADATA.HMR
      }
    }),
    new CompressionPlugin({
      regExp: /\.css$|\.html$|\.js$|\.map$/,
      threshold: 2 * 1024
    }),
    new LoaderOptionsPlugin({
      debug: false,
      options: {
        metadata: METADATA,
        tslint: {
          emitErrors: true,
          failOnHint: true,
          resourcePath: 'src',
        },

        node: {
          global: true,
          crypto: 'empty',
          process: false,
          module: false,
          clearImmediate: false,
          setImmediate: false
        }
      }
    })
  ]
});

var serverConfig = webpackMerge(commonConfig.SERVER, {
  devtool: 'sourcemap',
  output: {
    path: helpers.root('dist'),
    filename: '[name].api.bundle.js',
    sourceMapFilename: '[name].api.bundle.map',
    chunkFilename: '[id].api.chunk.js',
    libraryTarget: 'commonjs'
  },
  plugins: [
    ...commonPlugins,
    new CompressionPlugin({
      regExp: /\.json$|\.js$|\.map$/,
      threshold: 2 * 1024
    }),
    new LoaderOptionsPlugin({
      debug: false,
      options: {
        metadata: METADATA,
        tslint: {
          emitErrors: true,
          failOnHint: true,
          resourcePath: 'api',
        }
      }
    })
  ]
});

module.exports = [clientConfig, serverConfig];
